@@ -33,6 +33,7 @@
 #ifdef HAVE_MEEGO
 #include <mx-gtk/mx-gtk.h>
 #endif /* HAVE_MEEGO */
+#include <gio/gdesktopappinfo.h>
 
 #include <libempathy/empathy-utils.h>
 
@@ -65,7 +66,6 @@ typedef struct {
   GtkWidget *radiobutton_reuse;
 
   gboolean simple;
-  gboolean enabled;
 
   gboolean contains_pending_changes;
 
@@ -89,9 +89,6 @@ typedef struct {
   GtkWidget *param_account_widget;
   GtkWidget *param_password_widget;
 
-  gboolean automatic_change;
-  GtkWidget *remember_password_widget;
-
   /* Used only for IRC accounts */
   EmpathyIrcNetworkChooser *irc_network_chooser;
 
@@ -104,7 +101,6 @@ enum {
   PROP_SIMPLE,
   PROP_CREATING_ACCOUNT,
   PROP_OTHER_ACCOUNTS_EXIST,
-  PROP_ENABLED,
 };
 
 enum {
@@ -116,11 +112,58 @@ enum {
 
 static void account_widget_apply_and_log_in (EmpathyAccountWidget *);
 
+enum {
+  RESPONSE_LAUNCH
+};
+
 static guint signals[LAST_SIGNAL] = { 0 };
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountWidget)
 #define CHANGED_TIMEOUT 300
 
+#define DIGIT             "0-9"
+#define DIGITS            "(["DIGIT"]+)"
+#define ALPHA             "a-zA-Z"
+#define ALPHAS            "(["ALPHA"]+)"
+#define ALPHADIGIT        ALPHA DIGIT
+#define ALPHADIGITS       "(["ALPHADIGIT"]+)"
+#define ALPHADIGITDASH    ALPHA DIGIT "-"
+#define ALPHADIGITDASHS   "(["ALPHADIGITDASH"]*)"
+
+#define HOSTNUMBER        "("DIGITS"\\."DIGITS"\\."DIGITS"\\."DIGITS")"
+#define TOPLABEL          ALPHAS"|(["ALPHA"]" ALPHADIGITDASHS "["ALPHADIGIT"])"
+#define DOMAINLABEL       ALPHADIGITS"|(["ALPHADIGIT"]" ALPHADIGITDASHS \
+                                       "["ALPHADIGIT"])"
+#define HOSTNAME          "((" DOMAINLABEL "\\.)+" TOPLABEL ")"
+/* Based on http://www.ietf.org/rfc/rfc1738.txt (section 5) */
+#define HOST              "("HOSTNAME "|" HOSTNUMBER")"
+/* Based on http://www.ietf.org/rfc/rfc0822.txt (appendix D) */
+#define EMAIL_LOCALPART   "([^\\(\\)<>@,;:\\\\\"\\[\\]\\s]+)"
+
+/* UIN is digital according to the unofficial specification:
+ * http://iserverd.khstu.ru/docum_ext/icqv5.html#CTS
+ * 5 digits minimun according to http://en.wikipedia.org/wiki/ICQ#UIN */
+#define ICQ_USER_NAME     "(["DIGIT"]{5,})"
+/* Based on http://www.ietf.org/rfc/rfc2812.txt (section 2.3.1) */
+#define IRC_SPECIAL       "_\\[\\]{}\\\\|`^"
+#define IRC_USER_NAME     "(["ALPHA IRC_SPECIAL"]["ALPHADIGITDASH IRC_SPECIAL"]*)"
+/* Based on http://www.ietf.org/rfc/rfc4622.txt (section 2.2)
+ * We just exclude invalid characters to avoid ucschars and other redundant
+ * complexity */
+#define JABBER_USER_NAME  "([^@:'\"<>&\\s]+)"
+/* ID is an email according to the unofficial specification:
+ * http://www.hypothetic.org/docs/msn/general/names.php */
+#define MSN_USER_NAME     EMAIL_LOCALPART
+/* Based on the official help:
+ * http://help.yahoo.com/l/us/yahoo/edit/registration/edit-01.html */
+#define YAHOO_USER_NAME   "(["ALPHA"]["ALPHADIGIT"_\\.]{3,31})"
+
+#define ACCOUNT_REGEX_ICQ      "^"ICQ_USER_NAME"$"
+#define ACCOUNT_REGEX_IRC      "^"IRC_USER_NAME"$"
+#define ACCOUNT_REGEX_JABBER   "^"JABBER_USER_NAME"@"HOST"$"
+#define ACCOUNT_REGEX_MSN      "^"MSN_USER_NAME"@"HOST"$"
+#define ACCOUNT_REGEX_YAHOO    "^"YAHOO_USER_NAME"$"
+
 static void
 account_widget_set_control_buttons_sensitivity (EmpathyAccountWidget *self,
     gboolean sensitive)
@@ -140,6 +183,35 @@ account_widget_set_control_buttons_sensitivity (EmpathyAccountWidget *self,
     }
 }
 
+static void
+account_widget_set_entry_highlighting (GtkEntry *entry, gboolean highlight)
+{
+  GdkColor color;
+  GtkStyle *style;
+
+  g_return_if_fail (GTK_IS_ENTRY (entry));
+
+  style = gtk_widget_get_style (GTK_WIDGET (entry));
+
+  if (highlight)
+    {
+      color = style->bg[GTK_STATE_SELECTED];
+
+      /* Here we take the current theme colour and add it to
+       * the colour for white and average the two. This
+       * gives a colour which is inline with the theme but
+       * slightly whiter.
+       */
+      color.red = (color.red + (style->white).red) / 2;
+      color.green = (color.green + (style->white).green) / 2;
+      color.blue = (color.blue + (style->white).blue) / 2;
+
+      gtk_widget_modify_base (GTK_WIDGET (entry), GTK_STATE_NORMAL, &color);
+    }
+  else
+    gtk_widget_modify_base (GTK_WIDGET (entry), GTK_STATE_NORMAL, NULL);
+}
+
 static void
 account_widget_handle_control_buttons_sensitivity (EmpathyAccountWidget *self)
 {
@@ -161,9 +233,13 @@ account_widget_entry_changed_common (EmpathyAccountWidget *self,
   const gchar *str;
   const gchar *param_name;
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  gboolean prev_status;
+  gboolean curr_status;
 
   str = gtk_entry_get_text (entry);
   param_name = g_object_get_data (G_OBJECT (entry), "param_name");
+  prev_status = empathy_account_settings_parameter_is_valid (priv->settings,
+                                                             param_name);
 
   if (EMP_STR_EMPTY (str))
     {
@@ -185,21 +261,36 @@ account_widget_entry_changed_common (EmpathyAccountWidget *self,
           tp_strdiff (param_name, "password") ? str : "***");
       empathy_account_settings_set_string (priv->settings, param_name, str);
     }
+
+  curr_status = empathy_account_settings_parameter_is_valid (priv->settings,
+                                                             param_name);
+  if (curr_status != prev_status)
+    account_widget_set_entry_highlighting (entry, !curr_status);
 }
 
 static void
 account_widget_entry_changed_cb (GtkEditable *entry,
     EmpathyAccountWidget *self)
 {
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-
-  if (priv->automatic_change)
-    return;
-
   account_widget_entry_changed_common (self, GTK_ENTRY (entry), FALSE);
   empathy_account_widget_changed (self);
 }
 
+static void
+account_widget_entry_map_cb (GtkEntry *entry,
+    EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *param_name;
+  gboolean is_valid;
+
+  /* need to initialize input highlighting */
+  param_name = g_object_get_data (G_OBJECT (entry), "param_name");
+  is_valid = empathy_account_settings_parameter_is_valid (priv->settings,
+                                                          param_name);
+  account_widget_set_entry_highlighting (entry, !is_valid);
+}
+
 static void
 account_widget_int_changed_cb (GtkWidget *widget,
     EmpathyAccountWidget *self)
@@ -461,6 +552,8 @@ empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
 
       g_signal_connect (widget, "changed",
           G_CALLBACK (account_widget_entry_changed_cb), self);
+      g_signal_connect (widget, "map",
+          G_CALLBACK (account_widget_entry_map_cb), self);
     }
   else if (GTK_IS_TOGGLE_BUTTON (widget))
     {
@@ -758,23 +851,18 @@ account_widget_account_enabled_cb (GObject *source_object,
   g_object_unref (widget);
 }
 
-static void account_widget_applied_continue (EmpathyAccountWidget *widget,
-    gboolean reconnect_required);
-static void account_widget_applied_additional_cb (GObject *widget,
-    GAsyncResult *result, gpointer user_data);
-
 static void
 account_widget_applied_cb (GObject *source_object,
     GAsyncResult *res,
     gpointer user_data)
 {
   GError *error = NULL;
+  TpAccount *account;
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
-  gboolean reconnect_required;
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
-  empathy_account_settings_apply_finish (settings, res, &reconnect_required,
-      &error);
+  empathy_account_settings_apply_finish (settings, res, &error);
 
   if (error != NULL)
     {
@@ -783,44 +871,6 @@ account_widget_applied_cb (GObject *source_object,
       return;
     }
 
-  DEBUG ("First stage apply complete: reconnect_required = %s",
-      reconnect_required ? "yes" : "no");
-
-  /* if there is additional work for this widget, do that now */
-  if (widget->ui_details->additional_apply_async != NULL)
-    widget->ui_details->additional_apply_async (widget,
-        account_widget_applied_additional_cb,
-        GUINT_TO_POINTER (reconnect_required));
-  else
-    account_widget_applied_continue (widget, reconnect_required);
-}
-
-static void
-account_widget_applied_additional_cb (GObject *obj,
-    GAsyncResult *result,
-    gpointer user_data)
-{
-  EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (obj);
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
-  gboolean reconnect_required = GPOINTER_TO_UINT (user_data);
-
-  /* we don't care about the error status, the async method can report it,
-   * just get the result */
-  reconnect_required |= g_simple_async_result_get_op_res_gboolean (simple);
-
-  DEBUG ("Second stage apply complete: reconnect_required = %s",
-      reconnect_required ? "yes" : "no");
-
-  account_widget_applied_continue (widget, reconnect_required);
-}
-
-static void
-account_widget_applied_continue (EmpathyAccountWidget *widget,
-    gboolean reconnect_required)
-{
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
-  TpAccount *account;
-
   account = empathy_account_settings_get_account (priv->settings);
 
   if (account != NULL)
@@ -836,21 +886,23 @@ account_widget_applied_continue (EmpathyAccountWidget *widget,
               account_widget_account_enabled_cb, widget);
           g_signal_emit (widget, signals[ACCOUNT_CREATED], 0, account);
         }
-      else
+      else if (priv->enabled_checkbox != NULL)
         {
-          /* If the account was offline, we always want to try reconnecting,
-           * to give it a chance to connect if the previous params were wrong.
-           * tp_account_reconnect_async() won't do anything if the requested
-           * presence is offline anyway. */
-          if (tp_account_get_connection_status (account, NULL) ==
-              TP_CONNECTION_STATUS_DISCONNECTED)
-            reconnect_required = TRUE;
+          gboolean enabled_checked;
 
-          if (reconnect_required && tp_account_is_enabled (account)
-              && priv->enabled)
+          enabled_checked =
+#ifdef HAVE_MEEGO
+            mx_gtk_light_switch_get_active (
+                MX_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
+#else
+            gtk_toggle_button_get_active (
+                GTK_TOGGLE_BUTTON (priv->enabled_checkbox));
+#endif /* HAVE_MEEGO */
+
+          if (tp_account_is_enabled (account) && enabled_checked)
             {
               /* After having applied changes to a user account, we
-               * reconnect it if needed. This is done so the new
+               * automatically reconnect it. This is done so the new
                * information entered by the user is validated on the server. */
               tp_account_reconnect_async (account, NULL, NULL);
             }
@@ -965,6 +1017,100 @@ account_widget_build_generic (EmpathyAccountWidget *self,
         G_CALLBACK (account_widget_settings_ready_cb), self);
 }
 
+static void
+account_widget_launch_external_clicked (GtkWidget *button,
+    TpAccount *account)
+{
+  if (!tp_strdiff (tp_account_get_storage_provider (account),
+        "com.meego.libsocialweb"))
+    {
+      /* we know how to handle this external provider */
+      GDesktopAppInfo *app_info;
+      const gchar *args[3] = { NULL, };
+      GError *error = NULL;
+
+      app_info = g_desktop_app_info_new ("gnome-control-center.desktop");
+
+      if (app_info == NULL)
+        {
+          g_critical ("Could not locate 'gnome-control-center.desktop'");
+          return;
+        }
+
+      args[0] = g_app_info_get_commandline (G_APP_INFO (app_info));
+      args[1] = "bisho.desktop";
+      args[2] = NULL;
+
+      gdk_spawn_on_screen (gtk_widget_get_screen (button),
+          NULL, (gchar **) args, NULL,
+          G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
+      if (error != NULL)
+        {
+          g_critical ("Failed to launch editor: %s", error->message);
+          g_clear_error (&error);
+        }
+
+      g_object_unref (app_info);
+    }
+}
+
+static void
+account_widget_build_external (EmpathyAccountWidget *self,
+    EmpathyAccountSettings *settings)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  TpAccount *account = empathy_account_settings_get_account (settings);
+  GtkWidget *bar, *widget;
+  gchar *str;
+
+  self->ui_details->widget = gtk_vbox_new (FALSE, 6);
+  priv->table_common_settings = gtk_table_new (1, 2, FALSE);
+
+  if (!tp_strdiff (tp_account_get_storage_provider (account),
+        "com.meego.libsocialweb"))
+    {
+      /* we know how to handle this external provider */
+      str = g_strdup_printf (
+          _("The account %s is edited via My Web Accounts."),
+          empathy_account_settings_get_display_name (settings));
+    }
+  else
+    {
+      str = g_strdup_printf (
+          _("The account %s cannot be edited in Empathy."),
+          empathy_account_settings_get_display_name (settings));
+    }
+
+  widget = gtk_label_new (str);
+  gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
+  g_free (str);
+
+  bar = gtk_info_bar_new ();
+  gtk_info_bar_set_message_type (GTK_INFO_BAR (bar), GTK_MESSAGE_INFO);
+  gtk_container_add (
+      GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (bar))),
+      widget);
+  gtk_container_set_border_width (GTK_CONTAINER (bar), 6);
+
+  if (!tp_strdiff (tp_account_get_storage_provider (account),
+        "com.meego.libsocialweb"))
+    {
+      /* we know how to handle this external provider */
+      widget = gtk_info_bar_add_button (GTK_INFO_BAR (bar),
+          _("Launch My Web Accounts"), RESPONSE_LAUNCH);
+
+      g_signal_connect (widget, "clicked",
+          G_CALLBACK (account_widget_launch_external_clicked), account);
+    }
+
+  gtk_box_pack_start (GTK_BOX (self->ui_details->widget), bar,
+      FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (self->ui_details->widget),
+      priv->table_common_settings, FALSE, TRUE, 0);
+
+  gtk_widget_show_all (self->ui_details->widget);
+}
+
 static void
 account_widget_build_salut (EmpathyAccountWidget *self,
     const char *filename)
@@ -999,6 +1145,9 @@ account_widget_build_irc (EmpathyAccountWidget *self,
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 
+  empathy_account_settings_set_regex (priv->settings, "account",
+      ACCOUNT_REGEX_IRC);
+
   if (priv->simple)
     {
       priv->irc_network_chooser = empathy_account_widget_irc_build_simple (self,
@@ -1018,17 +1167,6 @@ account_widget_build_sip (EmpathyAccountWidget *self,
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
   empathy_account_widget_sip_build (self, filename,
     &priv->table_common_settings);
-
-  if (priv->simple)
-    {
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
-    }
-  else
-    {
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
-    }
 }
 
 static void
@@ -1037,6 +1175,9 @@ account_widget_build_msn (EmpathyAccountWidget *self,
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 
+  empathy_account_settings_set_regex (priv->settings, "account",
+      ACCOUNT_REGEX_MSN);
+
   if (priv->simple)
     {
       self->ui_details->gui = empathy_builder_get_file (filename,
@@ -1049,9 +1190,6 @@ account_widget_build_msn (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1068,9 +1206,6 @@ account_widget_build_msn (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1172,6 +1307,9 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
   is_gtalk = account_widget_is_gtalk (self);
   is_facebook = account_widget_is_facebook (self);
 
+  empathy_account_settings_set_regex (priv->settings, "account",
+      ACCOUNT_REGEX_JABBER);
+
   if (priv->simple && !is_gtalk && !is_facebook)
     {
       /* Simple widget for XMPP */
@@ -1197,9 +1335,6 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
     }
   else if (priv->simple && is_gtalk)
     {
@@ -1214,9 +1349,6 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_g_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_g_simple"));
     }
   else if (priv->simple && is_facebook)
     {
@@ -1233,9 +1365,6 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
       setup_facebook_id_widget (self, entry_id);
 
       self->ui_details->default_focus = g_strdup ("entry_id_fb_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_fb_simple"));
     }
   else
     {
@@ -1280,9 +1409,6 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
       self->ui_details->default_focus = g_strdup ("entry_id");
       priv->spinbutton_port = spinbutton_port;
 
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
-
       g_signal_connect (checkbutton_ssl, "toggled",
           G_CALLBACK (account_widget_jabber_ssl_toggled_cb),
           self);
@@ -1308,6 +1434,9 @@ account_widget_build_icq (EmpathyAccountWidget *self,
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
   GtkWidget *spinbutton_port;
 
+  empathy_account_settings_set_regex (priv->settings, "account",
+      ACCOUNT_REGEX_ICQ);
+
   if (priv->simple)
     {
       self->ui_details->gui = empathy_builder_get_file (filename,
@@ -1320,9 +1449,6 @@ account_widget_build_icq (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_uin_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1341,9 +1467,6 @@ account_widget_build_icq (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_uin");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1366,9 +1489,6 @@ account_widget_build_aim (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_screenname_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1386,9 +1506,6 @@ account_widget_build_aim (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_screenname");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1398,6 +1515,9 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 
+  empathy_account_settings_set_regex (priv->settings, "account",
+      ACCOUNT_REGEX_YAHOO);
+
   if (priv->simple)
     {
       self->ui_details->gui = empathy_builder_get_file (filename,
@@ -1410,9 +1530,6 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1424,6 +1541,7 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
       empathy_account_widget_handle_params (self,
           "entry_id", "account",
           "entry_password", "password",
+          "entry_server", "server",
           "entry_locale", "room-list-locale",
           "entry_charset", "charset",
           "spinbutton_port", "port",
@@ -1431,9 +1549,6 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1455,9 +1570,6 @@ account_widget_build_groupwise (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id_simple");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password_simple"));
     }
   else
     {
@@ -1474,9 +1586,6 @@ account_widget_build_groupwise (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-
-      priv->remember_password_widget = GTK_WIDGET (gtk_builder_get_object (
-              self->ui_details->gui, "remember_password"));
     }
 }
 
@@ -1513,7 +1622,6 @@ empathy_account_widget_enabled_cb (TpAccount *account,
     }
 }
 
-#if 0
 static void
 #ifdef HAVE_MEEGO
 account_widget_switch_flipped_cb (MxGtkLightSwitch *sw,
@@ -1540,7 +1648,6 @@ account_widget_enabled_toggled_cb (GtkToggleButton *toggle_button,
   tp_account_set_enabled_async (account, state,
       account_widget_account_enabled_cb, user_data);
 }
-#endif
 
 void
 empathy_account_widget_set_other_accounts_exist (EmpathyAccountWidget *self,
@@ -1577,9 +1684,6 @@ do_set_property (GObject *object,
       empathy_account_widget_set_other_accounts_exist (
           EMPATHY_ACCOUNT_WIDGET (object), g_value_get_boolean (value));
       break;
-    case PROP_ENABLED:
-      priv->enabled = g_value_get_boolean (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1611,9 +1715,6 @@ do_get_property (GObject *object,
     case PROP_OTHER_ACCOUNTS_EXIST:
       g_value_set_boolean (value, priv->other_accounts_exist);
       break;
-    case PROP_ENABLED:
-      g_value_set_boolean (value, priv->enabled);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1709,7 +1810,6 @@ static void
 add_enable_checkbox (EmpathyAccountWidget *self,
     TpAccount *account)
 {
-#if 0
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
 #ifdef HAVE_MEEGO
   GtkWidget *w;
@@ -1772,7 +1872,6 @@ add_enable_checkbox (EmpathyAccountWidget *self,
   g_signal_connect (G_OBJECT (priv->enabled_checkbox), "toggled",
       G_CALLBACK (account_widget_enabled_toggled_cb), self);
 #endif /* HAVE_MEEGO */
-#endif
 }
 
 #ifndef HAVE_MEEGO
@@ -1817,55 +1916,13 @@ add_register_buttons (EmpathyAccountWidget *self,
 }
 #endif /* HAVE_MEEGO */
 
-static void
-remember_password_toggled_cb (GtkToggleButton *button,
-    EmpathyAccountWidget *self)
-{
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-
-  if (gtk_toggle_button_get_active (button))
-    {
-      gtk_widget_set_sensitive (priv->param_password_widget, TRUE);
-    }
-  else
-    {
-      gtk_widget_set_sensitive (priv->param_password_widget, FALSE);
-      gtk_entry_set_text (GTK_ENTRY (priv->param_password_widget), "");
-      empathy_account_settings_unset (priv->settings, "password");
-    }
-}
-
-static void
-account_settings_password_retrieved_cb (GObject *object,
-    gpointer user_data)
-{
-  EmpathyAccountWidget *self = user_data;
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  const gchar *password = empathy_account_settings_get_string (
-      priv->settings, "password");
-
-  if (password != NULL)
-    {
-      /* We have to do this so that when we call gtk_entry_set_text,
-       * the ::changed callback doesn't think the user made the
-       * change. */
-      priv->automatic_change = TRUE;
-      gtk_entry_set_text (GTK_ENTRY (priv->param_password_widget), password);
-      priv->automatic_change = FALSE;
-    }
-
-  gtk_toggle_button_set_active (
-      GTK_TOGGLE_BUTTON (priv->remember_password_widget),
-      !EMP_STR_EMPTY (password));
-}
-
 static void
 do_constructed (GObject *obj)
 {
   EmpathyAccountWidget *self = EMPATHY_ACCOUNT_WIDGET (obj);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
   TpAccount *account;
-  const gchar *protocol, *cm_name;
+  TpStorageRestrictionFlags storage_restrictions;
   const gchar *display_name, *default_display_name;
   guint i = 0;
   struct {
@@ -1886,31 +1943,50 @@ do_constructed (GObject *obj)
     WIDGET (sofiasip, sip),
   };
 
-  cm_name = empathy_account_settings_get_cm (priv->settings);
-  protocol = empathy_account_settings_get_protocol (priv->settings);
+  account = empathy_account_settings_get_account (priv->settings);
 
-  for (i = 0 ; i < G_N_ELEMENTS (widgets); i++)
+  if (account != NULL)
+    storage_restrictions = tp_account_get_storage_restrictions (account);
+  else
+    storage_restrictions = 0;
+
+  /* Empathy can only edit accounts without the Cannot_Set_Parameters flag */
+  if (storage_restrictions & TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_PARAMETERS)
     {
-      if (!tp_strdiff (widgets[i].cm_name, cm_name) &&
-          !tp_strdiff (widgets[i].protocol, protocol))
-        {
-          gchar *filename;
+      DEBUG ("Account is provided by an external storage provider");
 
-          filename = empathy_file_lookup (widgets[i].file,
-              "libempathy-gtk");
-          widgets[i].func (self, filename);
-          g_free (filename);
-
-          break;
-        }
+      account_widget_build_external (self, priv->settings);
     }
-
-  if (i == G_N_ELEMENTS (widgets))
+  else
     {
-      gchar *filename = empathy_file_lookup (
-          "empathy-account-widget-generic.ui", "libempathy-gtk");
-      account_widget_build_generic (self, filename);
-      g_free (filename);
+      const gchar *protocol, *cm_name;
+
+      cm_name = empathy_account_settings_get_cm (priv->settings);
+      protocol = empathy_account_settings_get_protocol (priv->settings);
+
+      for (i = 0 ; i < G_N_ELEMENTS (widgets); i++)
+        {
+          if (!tp_strdiff (widgets[i].cm_name, cm_name) &&
+              !tp_strdiff (widgets[i].protocol, protocol))
+            {
+              gchar *filename;
+
+              filename = empathy_file_lookup (widgets[i].file,
+                  "libempathy-gtk");
+              widgets[i].func (self, filename);
+              g_free (filename);
+
+              break;
+            }
+        }
+
+      if (i == G_N_ELEMENTS (widgets))
+        {
+          gchar *filename = empathy_file_lookup (
+              "empathy-account-widget-generic.ui", "libempathy-gtk");
+          account_widget_build_generic (self, filename);
+          g_free (filename);
+        }
     }
 
   /* handle default focus */
@@ -1925,42 +2001,6 @@ do_constructed (GObject *obj)
           NULL);
     }
 
-  /* remember password */
-  if (priv->param_password_widget != NULL
-      && priv->remember_password_widget != NULL
-      && empathy_account_settings_supports_sasl (priv->settings))
-    {
-      if (priv->simple)
-        {
-          gtk_toggle_button_set_active (
-              GTK_TOGGLE_BUTTON (priv->remember_password_widget), TRUE);
-        }
-      else
-        {
-          gtk_toggle_button_set_active (
-              GTK_TOGGLE_BUTTON (priv->remember_password_widget),
-              !EMP_STR_EMPTY (empathy_account_settings_get_string (
-                      priv->settings, "password")));
-
-          /* The password might not have been retrieved from the
-           * keyring yet. We should update the remember password
-           * toggle button and the password entry when/if it is. */
-          g_signal_connect (priv->settings, "password-retrieved",
-              G_CALLBACK (account_settings_password_retrieved_cb), self);
-        }
-
-      g_signal_connect (priv->remember_password_widget, "toggled",
-          G_CALLBACK (remember_password_toggled_cb), self);
-
-      remember_password_toggled_cb (
-          GTK_TOGGLE_BUTTON (priv->remember_password_widget), self);
-    }
-  else if (priv->remember_password_widget != NULL
-      && !empathy_account_settings_supports_sasl (priv->settings))
-    {
-      gtk_widget_set_visible (priv->remember_password_widget, FALSE);
-    }
-
   /* dup and init the account-manager */
   priv->account_manager = tp_account_manager_dup ();
 
@@ -1969,7 +2009,9 @@ do_constructed (GObject *obj)
       account_manager_ready_cb, self);
 
   /* handle apply and cancel button */
-  if (!priv->simple)
+  if (!priv->simple &&
+      !(storage_restrictions &
+        TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_PARAMETERS))
     {
       GtkWidget *hbox = gtk_hbox_new (TRUE, 3);
       GtkWidget *image;
@@ -2018,8 +2060,6 @@ do_constructed (GObject *obj)
         account_widget_set_control_buttons_sensitivity (self, FALSE);
     }
 
-  account = empathy_account_settings_get_account (priv->settings);
-
   if (account != NULL)
     {
       g_signal_connect (account, "notify::enabled",
@@ -2029,15 +2069,21 @@ do_constructed (GObject *obj)
 #ifndef HAVE_MEEGO
   add_register_buttons (self, account);
 #endif /* HAVE_MEEGO */
-  add_enable_checkbox (self, account);
+
+  /* add the Enable checkbox to accounts that support it */
+  if (!(storage_restrictions & TP_STORAGE_RESTRICTION_FLAG_CANNOT_SET_ENABLED))
+    add_enable_checkbox (self, account);
 
   /* hook up to widget destruction to unref ourselves */
   g_signal_connect (self->ui_details->widget, "destroy",
       G_CALLBACK (account_widget_destroy_cb), self);
 
-  empathy_builder_unref_and_keep_widget (self->ui_details->gui,
-      self->ui_details->widget);
-  self->ui_details->gui = NULL;
+  if (self->ui_details->gui != NULL)
+    {
+      empathy_builder_unref_and_keep_widget (self->ui_details->gui,
+          self->ui_details->widget);
+      self->ui_details->gui = NULL;
+    }
 
   display_name = empathy_account_settings_get_display_name (priv->settings);
   default_display_name = empathy_account_widget_get_default_display_name (self);
@@ -2144,13 +2190,6 @@ empathy_account_widget_class_init (EmpathyAccountWidgetClass *klass)
   g_object_class_install_property (oclass, PROP_OTHER_ACCOUNTS_EXIST,
                   param_spec);
 
-  param_spec = g_param_spec_boolean ("enabled",
-      "Enabled",
-      "TRUE if this account widget is enabled",
-      FALSE,
-      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (oclass, PROP_ENABLED, param_spec);
-
   signals[HANDLE_APPLY] =
     g_signal_new ("handle-apply", G_TYPE_FROM_CLASS (klass),
         G_SIGNAL_RUN_LAST, 0, NULL, NULL,
