@@ -35,7 +35,8 @@ typedef enum {
 	EMPATHY_CONTACT_FEATURE_LOG = 1 << 2,
 	EMPATHY_CONTACT_FEATURE_EDIT = 1 << 3,
 	EMPATHY_CONTACT_FEATURE_INFO = 1 << 4,
-	EMPATHY_CONTACT_FEATURE_ALL = (1 << 5) - 1,
+	EMPATHY_CONTACT_FEATURE_FAVOURITE = 1 << 5,
+	EMPATHY_CONTACT_FEATURE_ALL = (1 << 6) - 1,
 } EmpathyContactFeatureFlags;
 
 GtkWidget * empathy_contact_menu_new           (EmpathyContact             *contact,
@@ -49,6 +50,8 @@ GtkWidget * empathy_contact_info_menu_item_new (EmpathyContact             *cont
 GtkWidget * empathy_contact_edit_menu_item_new (EmpathyContact             *contact);
 GtkWidget * empathy_contact_invite_menu_item_new (EmpathyContact *contact);
 GtkWidget * empathy_contact_file_transfer_menu_item_new (EmpathyContact    *contact);
+GtkWidget * empathy_contact_share_my_desktop_menu_item_new (EmpathyContact *contact);
+GtkWidget * empathy_contact_favourite_menu_item_new (EmpathyContact *contact);
 
 G_END_DECLS
 
