@@ -38,6 +38,7 @@
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
+#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-contact-groups.h>
 #include <libempathy/empathy-dispatcher.h>
@@ -46,8 +47,6 @@
 #include "empathy-individual-view.h"
 #include "empathy-individual-menu.h"
 #include "empathy-individual-store.h"
-#include "empathy-contact-dialogs.h"
-#include "empathy-individual-dialogs.h"
 #include "empathy-images.h"
 #include "empathy-linking-dialog.h"
 #include "empathy-cell-renderer-expander.h"
@@ -82,6 +81,12 @@ typedef struct
   guint expand_groups_idle_handler;
   /* owned string (group name) -> bool (whether to expand/contract) */
   GHashTable *expand_groups;
+
+  /* Auto scroll */
+  guint auto_scroll_timeout_id;
+  /* Distance between mouse pointer and the nearby border. Negative when
+     scrolling updards.*/
+  gint distance;
 } EmpathyIndividualViewPriv;
 
 typedef struct
@@ -157,7 +162,11 @@ individual_view_tooltip_destroy_cb (GtkWidget *widget,
 {
   EmpathyIndividualViewPriv *priv = GET_PRIV (view);
 
-  tp_clear_object (&priv->tooltip_widget);
+  if (priv->tooltip_widget != NULL)
+    {
+      DEBUG ("Tooltip destroyed");
+      tp_clear_object (&priv->tooltip_widget);
+    }
 }
 
 static gboolean
@@ -233,10 +242,10 @@ groups_change_group_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
-  FolksGroupDetails *group_details = FOLKS_GROUP_DETAILS (source);
+  FolksGroupable *groupable = FOLKS_GROUPABLE (source);
   GError *error = NULL;
 
-  folks_group_details_change_group_finish (group_details, result, &error);
+  folks_groupable_change_group_finish (groupable, result, &error);
   if (error != NULL)
     {
       g_warning ("failed to change group: %s", error->message);
@@ -367,16 +376,14 @@ real_drag_individual_received_cb (EmpathyIndividualView *self,
   if (!tp_strdiff (new_group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
     {
       /* Mark contact as favourite */
-      folks_favourite_details_set_is_favourite (
-          FOLKS_FAVOURITE_DETAILS (individual), TRUE);
+      folks_favourite_set_is_favourite (FOLKS_FAVOURITE (individual), TRUE);
       return;
     }
 
   if (!tp_strdiff (old_group, EMPATHY_INDIVIDUAL_STORE_FAVORITE))
     {
       /* Remove contact as favourite */
-      folks_favourite_details_set_is_favourite (
-          FOLKS_FAVOURITE_DETAILS (individual), FALSE);
+      folks_favourite_set_is_favourite (FOLKS_FAVOURITE (individual), FALSE);
 
       /* Don't try to remove it */
       old_group = NULL;
@@ -384,14 +391,14 @@ real_drag_individual_received_cb (EmpathyIndividualView *self,
 
   if (new_group != NULL)
     {
-      folks_group_details_change_group (FOLKS_GROUP_DETAILS (individual),
-          new_group, TRUE, groups_change_group_cb, NULL);
+      folks_groupable_change_group (FOLKS_GROUPABLE (individual), new_group, TRUE,
+          groups_change_group_cb, NULL);
     }
 
   if (old_group != NULL && action == GDK_ACTION_MOVE)
     {
-      folks_group_details_change_group (FOLKS_GROUP_DETAILS (individual),
-          old_group, FALSE, groups_change_group_cb, NULL);
+      folks_groupable_change_group (FOLKS_GROUPABLE (individual), old_group,
+          FALSE, groups_change_group_cb, NULL);
     }
 }
 
@@ -548,6 +555,34 @@ individual_view_drag_motion_cb (DragMotionData *data)
   return FALSE;
 }
 
+/* Minimum distance between the mouse pointer and a horizontal border when we
+   start auto scrolling. */
+#define AUTO_SCROLL_MARGIN_SIZE 20
+/* How far to scroll per one tick. */
+#define AUTO_SCROLL_PITCH       10
+
+static gboolean
+individual_view_auto_scroll_cb (EmpathyIndividualView *self)
+{
+	EmpathyIndividualViewPriv *priv = GET_PRIV (self);
+	GtkAdjustment         *adj;
+	gdouble                new_value;
+
+	adj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (self));
+
+	if (priv->distance < 0)
+		new_value = gtk_adjustment_get_value (adj) - AUTO_SCROLL_PITCH;
+	else
+		new_value = gtk_adjustment_get_value (adj) + AUTO_SCROLL_PITCH;
+
+	new_value = CLAMP (new_value, gtk_adjustment_get_lower (adj),
+		gtk_adjustment_get_upper (adj) - gtk_adjustment_get_page_size (adj));
+
+	gtk_adjustment_set_value (adj, new_value);
+
+	return TRUE;
+}
+
 static gboolean
 individual_view_drag_motion (GtkWidget *widget,
     GdkDragContext *context,
@@ -565,10 +600,32 @@ individual_view_drag_motion (GtkWidget *widget,
   gboolean is_different = FALSE;
   gboolean cleanup = TRUE;
   gboolean retval = TRUE;
+  GtkAllocation allocation;
 
   priv = GET_PRIV (EMPATHY_INDIVIDUAL_VIEW (widget));
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 
+
+  if (priv->auto_scroll_timeout_id != 0)
+    {
+      g_source_remove (priv->auto_scroll_timeout_id);
+      priv->auto_scroll_timeout_id = 0;
+    }
+
+  gtk_widget_get_allocation (widget, &allocation);
+
+  if (y < AUTO_SCROLL_MARGIN_SIZE ||
+      y > (allocation.height - AUTO_SCROLL_MARGIN_SIZE))
+    {
+      if (y < AUTO_SCROLL_MARGIN_SIZE)
+        priv->distance = MIN (-y, -1);
+      else
+        priv->distance = MAX (allocation.height - y, 1);
+
+      priv->auto_scroll_timeout_id = g_timeout_add (10 * ABS (priv->distance),
+          (GSourceFunc) individual_view_auto_scroll_cb, widget);
+    }
+
   is_row = gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
       x, y, &path, NULL, NULL, NULL);
 
@@ -623,8 +680,7 @@ individual_view_drag_motion (GtkWidget *widget,
         }
 
       if (individual != NULL &&
-          folks_presence_details_is_online (
-              FOLKS_PRESENCE_DETAILS (individual)) &&
+          folks_individual_is_online (individual) &&
           (caps & EMPATHY_CAPABILITIES_FT))
         {
           gdk_drag_status (context, GDK_ACTION_COPY, time_);
@@ -849,6 +905,8 @@ individual_view_popup_menu_idle_cb (gpointer user_data)
       gtk_widget_show (menu);
       gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, data->button,
           data->time);
+      g_object_ref_sink (menu);
+      g_object_unref (menu);
     }
 
   g_slice_free (MenuPopupData, data);
@@ -880,7 +938,7 @@ individual_view_key_press_event_cb (EmpathyIndividualView *view,
     GdkEventKey *event,
     gpointer user_data)
 {
-  if (event->keyval == GDK_Menu)
+  if (event->keyval == GDK_KEY_Menu)
     {
       MenuPopupData *data;
 
@@ -889,25 +947,6 @@ individual_view_key_press_event_cb (EmpathyIndividualView *view,
       data->button = 0;
       data->time = event->time;
       g_idle_add (individual_view_popup_menu_idle_cb, data);
-    } else if (event->keyval == GDK_KEY_F2) {
-        FolksIndividual *individual;
-        EmpathyContact *contact;
-
-        g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (view), FALSE);
-
-        individual = empathy_individual_view_dup_selected (view);
-        if (individual == NULL)
-            return FALSE;
-
-        contact = empathy_contact_dup_from_folks_individual (individual);
-        if (contact == NULL) {
-            g_object_unref (individual);
-            return FALSE;
-        }
-        empathy_contact_edit_dialog_show (contact, NULL);
-
-        g_object_unref (individual);
-        g_object_unref (contact);
     }
 
   return FALSE;
@@ -1240,10 +1279,7 @@ individual_view_start_search_cb (EmpathyIndividualView *view,
   if (priv->search_widget == NULL)
     return FALSE;
 
-  if (gtk_widget_get_visible (GTK_WIDGET (priv->search_widget)))
-    gtk_widget_grab_focus (GTK_WIDGET (priv->search_widget));
-  else
-    gtk_widget_show (GTK_WIDGET (priv->search_widget));
+  empathy_individual_view_start_search (view);
 
   return TRUE;
 }
@@ -1331,8 +1367,7 @@ individual_view_search_key_navigation_cb (GtkWidget *search,
   GdkEventKey *eventkey = ((GdkEventKey *) event);
   gboolean ret = FALSE;
 
-  if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down
-      || eventkey->keyval == GDK_KEY_F2)
+  if (eventkey->keyval == GDK_KEY_Up || eventkey->keyval == GDK_KEY_Down)
     {
       GdkEvent *new_event;
 
@@ -1617,7 +1652,6 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
   EmpathyLiveSearch *live = EMPATHY_LIVE_SEARCH (priv->search_widget);
   const gchar *str;
   GList *personas, *l;
-  gboolean is_favorite, contains_interesting_persona = FALSE;
 
   /* We're only giving the visibility wrt filtering here, not things like
    * presence. */
@@ -1627,39 +1661,24 @@ individual_view_is_visible_individual (EmpathyIndividualView *self,
       return FALSE;
     }
 
-  /* Hide all individuals which consist entirely of uninteresting personas */
-  personas = folks_individual_get_personas (individual);
-  for (l = personas; l; l = l->next)
-    {
-      if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        {
-          contains_interesting_persona = TRUE;
-          break;
-        }
-    }
-
-  if (contains_interesting_persona == FALSE)
-    return FALSE;
-
-  is_favorite = folks_favourite_details_get_is_favourite (
-      FOLKS_FAVOURITE_DETAILS (individual));
   if (is_searching == FALSE)
-    return (priv->show_offline || is_online || is_favorite);
+    return (priv->show_offline || is_online);
 
   /* check alias name */
-  str = folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (individual));
+  str = folks_individual_get_alias (individual);
 
   if (empathy_live_search_match (live, str))
     return TRUE;
 
   /* check contact id, remove the @server.com part */
+  personas = folks_individual_get_personas (individual);
   for (l = personas; l; l = l->next)
     {
       const gchar *p;
       gchar *dup_str = NULL;
       gboolean visible;
 
-      if (!empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
+      if (!TPF_IS_PERSONA (l->data))
         continue;
 
       str = folks_persona_get_display_id (l->data);
@@ -1926,7 +1945,7 @@ individual_view_dispose (GObject *object)
 
   tp_clear_object (&priv->store);
   tp_clear_object (&priv->filter);
-  tp_clear_object (&priv->tooltip_widget);
+  tp_clear_pointer (&priv->tooltip_widget, gtk_widget_destroy);
 
   empathy_individual_view_set_live_search (view, NULL);
 
@@ -2166,29 +2185,6 @@ empathy_individual_view_dup_selected (EmpathyIndividualView *view)
   return individual;
 }
 
-EmpathyIndividualManagerFlags
-empathy_individual_view_get_flags (EmpathyIndividualView *view)
-{
-  EmpathyIndividualViewPriv *priv;
-  GtkTreeSelection *selection;
-  GtkTreeIter iter;
-  GtkTreeModel *model;
-  EmpathyIndividualFeatureFlags flags;
-
-  g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (view), 0);
-
-  priv = GET_PRIV (view);
-
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-    return 0;
-
-  gtk_tree_model_get (model, &iter,
-      EMPATHY_INDIVIDUAL_STORE_COL_FLAGS, &flags, -1);
-
-  return flags;
-}
-
 gchar *
 empathy_individual_view_get_selected_group (EmpathyIndividualView *view,
     gboolean *is_fake_group)
@@ -2226,52 +2222,19 @@ empathy_individual_view_get_selected_group (EmpathyIndividualView *view,
   return name;
 }
 
-enum
-{
-  REMOVE_DIALOG_RESPONSE_CANCEL = 0,
-  REMOVE_DIALOG_RESPONSE_DELETE,
-  REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK,
-};
-
-static int
+static gboolean
 individual_view_remove_dialog_show (GtkWindow *parent,
     const gchar *message,
-    const gchar *secondary_text,
-    gboolean block_button,
-    GdkPixbuf *avatar)
+    const gchar *secondary_text)
 {
   GtkWidget *dialog;
   gboolean res;
 
   dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", message);
-
-  if (avatar != NULL)
-    {
-      GtkWidget *image = gtk_image_new_from_pixbuf (avatar);
-      gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
-      gtk_widget_show (image);
-    }
-
-  if (block_button)
-    {
-      GtkWidget *button;
-
-      /* gtk_dialog_add_button() doesn't allow us to pass a string with a
-       * mnemonic so we have to create the button manually. */
-      button = gtk_button_new_with_mnemonic (
-          _("Delete and _Block"));
-
-      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
-          REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK);
-
-      gtk_widget_show (button);
-    }
-
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-      GTK_STOCK_CANCEL, REMOVE_DIALOG_RESPONSE_CANCEL,
-      GTK_STOCK_DELETE, REMOVE_DIALOG_RESPONSE_DELETE, NULL);
-
+      GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
+      GTK_STOCK_DELETE, GTK_RESPONSE_YES, NULL);
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
       "%s", secondary_text);
 
@@ -2280,7 +2243,7 @@ individual_view_remove_dialog_show (GtkWindow *parent,
   res = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
 
-  return res;
+  return (res == GTK_RESPONSE_YES);
 }
 
 static void
@@ -2300,7 +2263,7 @@ individual_view_group_remove_activate_cb (GtkMenuItem *menuitem,
           group);
       parent = empathy_get_toplevel_window (GTK_WIDGET (view));
       if (individual_view_remove_dialog_show (parent, _("Removing group"),
-              text, FALSE, NULL) == REMOVE_DIALOG_RESPONSE_DELETE)
+              text))
         {
           EmpathyIndividualManager *manager =
               empathy_individual_manager_dup_singleton ();
@@ -2334,7 +2297,6 @@ empathy_individual_view_get_group_menu (EmpathyIndividualView *view)
   if (!group || is_fake_group)
     {
       /* We can't alter fake groups */
-      g_free (group);
       return NULL;
     }
 
@@ -2369,69 +2331,6 @@ empathy_individual_view_get_group_menu (EmpathyIndividualView *view)
   return menu;
 }
 
-static void
-got_avatar (GObject *source_object,
-    GAsyncResult *result,
-    gpointer user_data)
-{
-  FolksIndividual *individual = FOLKS_INDIVIDUAL (source_object);
-  EmpathyIndividualView *view = user_data;
-  EmpathyIndividualManager *manager;
-  GdkPixbuf *avatar;
-  gchar *text;
-  GtkWindow *parent;
-  gboolean can_block;
-  GError *error = NULL;
-  gint res;
-
-  avatar = empathy_pixbuf_avatar_from_individual_scaled_finish (individual,
-      result, &error);
-
-  if (error != NULL)
-    {
-      DEBUG ("Could not get avatar: %s", error->message);
-      g_error_free (error);
-    }
-
-  /* We couldn't retrieve the avatar, but that isn't a fatal error,
-   * so we still display the remove dialog. */
-
-  manager = empathy_individual_manager_dup_singleton ();
-  can_block = empathy_individual_manager_supports_blocking (manager,
-      individual);
-
-  parent = empathy_get_toplevel_window (GTK_WIDGET (view));
-  text =
-      g_strdup_printf (
-      _("Are you sure you want to remove '%s' from your contacts?"),
-      folks_alias_details_get_alias (FOLKS_ALIAS_DETAILS (individual)));
-
-  res = individual_view_remove_dialog_show (parent, _("Removing contact"),
-      text, can_block, avatar);
-
-  if (res == REMOVE_DIALOG_RESPONSE_DELETE ||
-      res == REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK)
-    {
-      gboolean abusive;
-
-      if (res == REMOVE_DIALOG_RESPONSE_DELETE_AND_BLOCK)
-        {
-          if (!empathy_block_individual_dialog_show (parent, individual,
-               avatar, &abusive))
-            goto finally;
-
-          empathy_individual_manager_set_blocked (manager, individual,
-              TRUE, abusive);
-        }
-
-      empathy_individual_manager_remove (manager, individual, "");
-    }
-
- finally:
-  g_free (text);
-  g_object_unref (manager);
-}
-
 static void
 individual_view_remove_activate_cb (GtkMenuItem *menuitem,
     EmpathyIndividualView *view)
@@ -2442,8 +2341,25 @@ individual_view_remove_activate_cb (GtkMenuItem *menuitem,
 
   if (individual != NULL)
     {
-      empathy_pixbuf_avatar_from_individual_scaled_async (individual,
-          48, 48, NULL, got_avatar, view);
+      gchar *text;
+      GtkWindow *parent;
+
+      parent = empathy_get_toplevel_window (GTK_WIDGET (view));
+      text =
+          g_strdup_printf (_
+          ("Do you really want to remove the contact '%s'?"),
+          folks_individual_get_alias (individual));
+      if (individual_view_remove_dialog_show (parent, _("Removing contact"),
+              text))
+        {
+          EmpathyIndividualManager *manager;
+
+          manager = empathy_individual_manager_dup_singleton ();
+          empathy_individual_manager_remove (manager, individual, "");
+          g_object_unref (G_OBJECT (manager));
+        }
+
+      g_free (text);
       g_object_unref (individual);
     }
 }
@@ -2469,7 +2385,8 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   GtkWidget *menu = NULL;
   GtkWidget *item;
   GtkWidget *image;
-  EmpathyIndividualManagerFlags flags;
+  gboolean can_remove = FALSE;
+  GList *l;
 
   g_return_val_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (view), NULL);
 
@@ -2477,16 +2394,31 @@ empathy_individual_view_get_individual_menu (EmpathyIndividualView *view)
   if (individual == NULL)
     return NULL;
 
-  flags = empathy_individual_view_get_flags (view);
+  /* If any of the Individual's personas can be removed, add an option to
+   * remove. This will act as a best-effort option. If any Personas cannot be
+   * removed from the server, then this option will just be inactive upon
+   * subsequent menu openings */
+  for (l = folks_individual_get_personas (individual); l != NULL; l = l->next)
+    {
+      FolksPersona *persona = FOLKS_PERSONA (l->data);
+      FolksPersonaStore *store = folks_persona_get_store (persona);
+      FolksMaybeBool maybe_can_remove =
+          folks_persona_store_get_can_remove_personas (store);
+
+      if (maybe_can_remove == FOLKS_MAYBE_BOOL_TRUE)
+        {
+          can_remove = TRUE;
+          break;
+        }
+    }
 
   menu = empathy_individual_menu_new (individual, priv->individual_features);
 
   /* Remove contact */
-  if (priv->view_features &
-      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_REMOVE &&
-      flags & EMPATHY_INDIVIDUAL_MANAGER_CAN_REMOVE)
+  if ((priv->view_features &
+      EMPATHY_INDIVIDUAL_VIEW_FEATURE_INDIVIDUAL_REMOVE) &&
+      can_remove)
     {
-
       /* create the menu if required, or just add a separator */
       if (menu == NULL)
         menu = gtk_menu_new ();
@@ -2693,3 +2625,17 @@ empathy_individual_view_set_store (EmpathyIndividualView *self,
           G_CALLBACK (individual_view_store_row_deleted_cb), self, 0);
     }
 }
+
+void
+empathy_individual_view_start_search (EmpathyIndividualView *self)
+{
+  EmpathyIndividualViewPriv *priv = GET_PRIV (self);
+
+  g_return_if_fail (EMPATHY_IS_INDIVIDUAL_VIEW (self));
+  g_return_if_fail (priv->search_widget != NULL);
+
+  if (gtk_widget_get_visible (GTK_WIDGET (priv->search_widget)))
+    gtk_widget_grab_focus (GTK_WIDGET (priv->search_widget));
+  else
+    gtk_widget_show (GTK_WIDGET (priv->search_widget));
+}
